/*
** File: model_adpexrc6tadv.c
** Project: ADMXRC3 module driver
** Purpose: ADPE-XRC-6T-ADV specific code, including the function to bootstrap
**          the device context.
**
** (C) Copyright Alpha Data 2012
*/

#include <df.h>

#include "device.h"
#include "info.h"
#include "model_boot.h"
#include <core/adpexrc6tadv.h>

#define NUM_BANK_PER_TARGET 4U
static const unsigned int numBankPerTarget = NUM_BANK_PER_TARGET;
static const unsigned int numBank = NUM_BANK_PER_TARGET * ADPEXRC6TADVCTL_NUM_TARGET_FPGA;

static ADMXRC3_STATUS
getBankInfoControl(
	Admxrc3DeviceContext* pDevCtx,
  unsigned int bankIndex,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETBANKINFO* pIoctl)
{
  VpdAdpexrc6tadvRev0* pVpd = (VpdAdpexrc6tadvRev0*)pVpdBuffer;
  uint8_t order;
  unsigned int targetIndex;

  if (bankIndex >= numBank) {
    return ADMXRC3_INVALID_INDEX;
  }
  targetIndex = bankIndex / numBankPerTarget;
  bankIndex = bankIndex & (numBankPerTarget - 1);

  order = pVpd->target[targetIndex].sdram[bankIndex].order;
  pIoctl->out.typeMask = ADMXRC3_BANK_SDRAM_DDR3;
  pIoctl->out.physicalDataWidth = 32;
  pIoctl->out.physicalEccWidth = 0;
  pIoctl->out.maximumFrequency = (uint64_t)pVpd->target[targetIndex].sdram[bankIndex].speed * 100000;
  pIoctl->out.minimumFrequency = 303030303U;
  pIoctl->out.connectivityMask = 0x1U << targetIndex;
  pIoctl->out.physicalSize = (uint64_t)0x1U << order;
  pIoctl->out.bPresent = (order != 0);

  return ADMXRC3_SUCCESS;
}

static ADMXRC3_STATUS
getBankInfoTarget(
	Admxrc3DeviceContext* pDevCtx,
  unsigned int bankIndex,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETBANKINFO* pIoctl)
{
  VpdAdpexrc6tadvRev0* pVpd = (VpdAdpexrc6tadvRev0*)pVpdBuffer;
  uint8_t order;
  unsigned int targetIndex;

  if (bankIndex >= numBankPerTarget) {
    return ADMXRC3_INVALID_INDEX;
  }
  targetIndex = (pDevCtx->info.model == CoreModelAdpexrc6tadvTarget0) ? 0 : 1;
  order = pVpd->target[targetIndex].sdram[bankIndex].order;
  pIoctl->out.typeMask = ADMXRC3_BANK_SDRAM_DDR3;
  pIoctl->out.physicalDataWidth = 32;
  pIoctl->out.physicalEccWidth = 0;
  pIoctl->out.maximumFrequency = (uint64_t)pVpd->target[targetIndex].sdram[bankIndex].speed * 100000;
  pIoctl->out.minimumFrequency = 303030303U;
  pIoctl->out.connectivityMask = 0x1U;
  pIoctl->out.physicalSize = (uint64_t)0x1U << order;
  pIoctl->out.bPresent = (order != 0);

  return ADMXRC3_SUCCESS;
}

static void
getCardInfoControl(
	Admxrc3DeviceContext* pDevCtx,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETCARDINFO* pIoctl)
{
  VpdAdpexrc6tadvRev0* pVpd = (VpdAdpexrc6tadvRev0*)pVpdBuffer;
  unsigned int i, targetIndex, bankIndex;

  pIoctl->out.serialNumber = pVpd->board.serial;
  pIoctl->out.model = ADMXRC3_MODEL_ADPEXRC6TADV_C;
  pIoctl->out.numMemoryBank = 8;
  pIoctl->out.memoryBankPresent = 0;
  for (targetIndex = 0; targetIndex < ADPEXRC6TADVCTL_NUM_TARGET_FPGA; targetIndex++) {
    for (i = 0; i < numBankPerTarget; i++) {
      if (0 != pVpd->target[targetIndex].sdram[i].order) {
        /* Bank is fitted */
        bankIndex = i + targetIndex * numBankPerTarget;
        pIoctl->out.memoryBankPresent |= (0x1U << bankIndex);
      }
    }
  }
}

static void
getCardInfoTarget(
	Admxrc3DeviceContext* pDevCtx,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETCARDINFO* pIoctl)
{
  VpdAdpexrc6tadvRev0* pVpd = (VpdAdpexrc6tadvRev0*)pVpdBuffer;
  unsigned int i, targetIndex;

  targetIndex = (pDevCtx->info.model == CoreModelAdpexrc6tadvTarget0) ? 0 : 1;
  pIoctl->out.serialNumber = pVpd->board.serial;
  pIoctl->out.model = ADMXRC3_MODEL_ADPEXRC6TADV_T;
  pIoctl->out.numMemoryBank = numBankPerTarget;
  pIoctl->out.memoryBankPresent = 0;
  for (i = 0; i < numBankPerTarget; i++) {
    if (0 != pVpd->target[targetIndex].sdram[i].order) {
      /* Bank is fitted */
      pIoctl->out.memoryBankPresent |= (0x1U << i);
    }
  }
}

static ADMXRC3_STATUS
getFpgaInfoControl(
	Admxrc3DeviceContext* pDevCtx,
  unsigned int targetIndex,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETFPGAINFO* pIoctl)
{
  VpdAdpexrc6tadvRev0* pVpd = (VpdAdpexrc6tadvRev0*)pVpdBuffer;
  ADMXRC3_PACKAGE_TYPE package;
  ADMXRC3_FPGA_TYPE fpgaType;
  char speed[8];
  char stepping[8];

  if (targetIndex >= pDevCtx->info.numTargetFpga) {
    return ADMXRC3_INVALID_INDEX;
  }

  package = ADMXRC3_PACKAGE_MAKEFF(1759);
  fpgaType = (ADMXRC3_FPGA_TYPE)pVpd->target[targetIndex].fpga.device;

  dfAssert(sizeof(char) == sizeof(uint8_t));

  dfStrCpy(speed, DF_ARRAY_LENGTH(speed), (const char*)pVpd->target[targetIndex].fpga.speedGrade);
  switch (pVpd->board.tempGrade) {
  case ADPEXRC6TADV_VPD_TEMPGRADE_COMMERCIAL:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "C");
    break;

  case ADPEXRC6TADV_VPD_TEMPGRADE_INDUSTRIAL:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "I");
    break;

  case ADPEXRC6TADV_VPD_TEMPGRADE_EXTENDED:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "E");
    break;

  default:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "?");
    break;
  }

  dfStrCpy(stepping, DF_ARRAY_LENGTH(stepping), (const char*)pVpd->target[targetIndex].fpga.stepping);

  pIoctl->out.flags = ADMXRC3_FPGA_SPEEDVALID;
  setupIoctlsGetFpgaInfo(pDevCtx, pIoctl, fpgaType, package, speed, stepping, TRUE);

  return ADMXRC3_SUCCESS;
}

static ADMXRC3_STATUS
getFpgaInfoTarget(
	Admxrc3DeviceContext* pDevCtx,
  unsigned int targetIndex,
  const void* pVpdBuffer,
  IOCTLS_ADMXRC3_GETFPGAINFO* pIoctl)
{
  VpdAdpexrc6tadvRev0* pVpd = (VpdAdpexrc6tadvRev0*)pVpdBuffer;
  ADMXRC3_PACKAGE_TYPE package;
  ADMXRC3_FPGA_TYPE fpgaType;
  char speed[8];
  char stepping[8];

  if (targetIndex >= pDevCtx->info.numTargetFpga) {
    return ADMXRC3_INVALID_INDEX;
  }

  targetIndex = (pDevCtx->info.model == CoreModelAdpexrc6tadvTarget0) ? 0 : 1;

  package = ADMXRC3_PACKAGE_MAKEFF(1759);
  fpgaType = (ADMXRC3_FPGA_TYPE)pVpd->target[targetIndex].fpga.device;

  dfAssert(sizeof(char) == sizeof(uint8_t));

  dfStrCpy(speed, DF_ARRAY_LENGTH(speed), (const char*)pVpd->target[targetIndex].fpga.speedGrade);
  switch (pVpd->board.tempGrade) {
  case ADPEXRC6TADV_VPD_TEMPGRADE_COMMERCIAL:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "C");
    break;

  case ADPEXRC6TADV_VPD_TEMPGRADE_INDUSTRIAL:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "I");
    break;

  case ADPEXRC6TADV_VPD_TEMPGRADE_EXTENDED:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "E");
    break;

  default:
    dfStrCat(speed, DF_ARRAY_LENGTH(speed), "?");
    break;
  }

  dfStrCpy(stepping, DF_ARRAY_LENGTH(stepping), (const char*)pVpd->target[targetIndex].fpga.stepping);

  pIoctl->out.flags = ADMXRC3_FPGA_SPEEDVALID | ADMXRC3_FPGA_NOTCONFIGURABLE;
  setupIoctlsGetFpgaInfo(pDevCtx, pIoctl, fpgaType, package, speed, stepping, TRUE);

  return ADMXRC3_SUCCESS;
}

void
admxrc3BootstrapAdpexrc6tadv(
	Admxrc3DeviceContext* pDevCtx)
{
  dfDebugPrint(1, ("admxrc3BootstrapAdpexrc6tadv: entered\n"));

  pDevCtx->methods.pGetBankInfo = (pDevCtx->info.model == CoreModelAdpexrc6tadvControl) ? getBankInfoControl : getBankInfoTarget;
  pDevCtx->methods.pGetCardInfo = (pDevCtx->info.model == CoreModelAdpexrc6tadvControl) ? getCardInfoControl : getCardInfoTarget;
  pDevCtx->methods.pGetFpgaInfo = (pDevCtx->info.model == CoreModelAdpexrc6tadvControl) ? getFpgaInfoControl : getFpgaInfoTarget;
  pDevCtx->bReverseSelectMapData = FALSE;
  pDevCtx->localAddressLimit = (uint64_t)(0x1ULL << 39U);
  pDevCtx->localWidth = 16U;
}
